@@ -14,68 +14,50 @@ import seaborn as sns
 import matplotlib.pyplot as plt
 get_ipython().magic('matplotlib inline')
 sns.set(style="white", color_codes=True)
-import pdb
 import time
-import pprint
 import matplotlib
 matplotlib.style.use('ggplot')
 from sklearn.pipeline import make_pipeline
 from sklearn.model_selection import RandomizedSearchCV
-from sklearn.calibration import CalibratedClassifierCV
 
 from sklearn import model_selection
 import warnings
 warnings.filterwarnings("ignore")
 from sklearn import metrics
-from sklearn.neighbors import KNeighborsClassifier, KNeighborsRegressor
 from sklearn.model_selection import StratifiedKFold
-from sklearn.preprocessing import StandardScaler
-from sklearn.ensemble import RandomForestClassifier, RandomForestRegressor, BaggingRegressor
-from sklearn.metrics import roc_auc_score, classification_report, confusion_matrix
-from sklearn.metrics import f1_score, precision_score, recall_score,accuracy_score,make_scorer
-from sklearn.model_selection import train_test_split, StratifiedShuffleSplit, KFold, StratifiedKFold
+from sklearn.metrics import roc_auc_score
+from sklearn.metrics import f1_score, precision_score, recall_score
+from sklearn.model_selection import KFold, StratifiedKFold, train_test_split
 from sklearn.metrics import mean_squared_error, mean_absolute_error
-from sklearn.preprocessing import LabelEncoder
 
 ### For NLP problems
-from sklearn.feature_extraction.text import CountVectorizer, TfidfTransformer, TfidfVectorizer
-from textblob import TextBlob, Word
+from sklearn.feature_extraction.text import CountVectorizer, TfidfVectorizer
+from textblob import TextBlob
 from sklearn.feature_extraction.text import TfidfVectorizer
 import regex as re
 import string
 
 #### For Classification problems
 from sklearn.naive_bayes import MultinomialNB
-from sklearn.linear_model import LogisticRegression
 from sklearn import metrics
 
 #### For Regression problems
 from sklearn.model_selection import train_test_split
-from sklearn.pipeline import Pipeline
-from sklearn.ensemble import RandomForestRegressor
 from sklearn.metrics import mean_squared_error
 
 
-from scipy.stats import multivariate_normal
 from sklearn.metrics import roc_auc_score
-from sklearn.metrics import f1_score, precision_score, recall_score,accuracy_score,make_scorer
-from sklearn.model_selection import train_test_split, StratifiedShuffleSplit, KFold, StratifiedKFold
-from sklearn.preprocessing import StandardScaler, MinMaxScaler
+from sklearn.metrics import f1_score, precision_score, recall_score
+from sklearn.model_selection import KFold, StratifiedKFold, train_test_split
 from sklearn import model_selection, metrics   #Additional sklearn functions
 from sklearn.model_selection import GridSearchCV   #Performing grid search
-from sklearn.model_selection import cross_val_predict, cross_val_score
-from sklearn.model_selection import KFold, StratifiedKFold, cross_val_score
-from sklearn.metrics import auc, roc_auc_score, roc_curve, precision_recall_curve
-from sklearn.svm import SVC
+from sklearn.model_selection import KFold, StratifiedKFold
+from sklearn.metrics import roc_auc_score
 
 import copy
-import pdb
 from itertools import cycle
-from collections import defaultdict, Counter
-import operator
-from scipy import interp
+from collections import Counter
 import nltk
-from nltk.corpus import stopwords
 nltk.download("popular")
 ############################################################################
 # define a function that accepts a vectorizer and calculates its accuracy
@@ -125,25 +107,15 @@ def tokenize_test_by_metric(model, X_train, X_cv, y_train, y_cv,
 ###########     N  L  P    F  U  N C  T  I  O   N  S       #####################
 ################################################################################
 '''Features'''
-from sklearn.preprocessing import LabelEncoder
 from sklearn.feature_extraction.text import TfidfVectorizer, CountVectorizer
 from sklearn.decomposition import TruncatedSVD
-from sklearn.preprocessing import label_binarize
 
 '''Classifiers'''
-from sklearn.dummy import DummyClassifier
-from sklearn.linear_model import SGDClassifier
 from sklearn.naive_bayes import GaussianNB
-from sklearn.tree import DecisionTreeClassifier
-from sklearn.ensemble import RandomForestClassifier, AdaBoostClassifier
-from sklearn.neighbors import KNeighborsClassifier
-from sklearn.multiclass import OneVsRestClassifier
 
 '''Metrics/Evaluation'''
 from sklearn.model_selection import train_test_split
-from sklearn.model_selection import GridSearchCV
-from sklearn.metrics import accuracy_score, precision_score, recall_score, f1_score, roc_curve, auc, confusion_matrix
-from scipy import interp
+from sklearn.metrics import f1_score, precision_score, recall_score
 from itertools import cycle
 
 '''Plotting'''
@@ -162,15 +134,15 @@ pd.options.display.float_format = '{:,.2f}'.format
 ####  R O B   S A L G A D O    robert.salgado@gmail.com     Thank YOU!
 # https://github.com/robsalgado/personal_data_science_projects/tree/master/mulitclass_text_class
 ################################################################################
-import itertools, string, operator, re, unicodedata, nltk
-from operator import itemgetter
+import nltk
+import re
+import string
+import unicodedata
 from nltk.corpus import wordnet
 from nltk.stem import WordNetLemmatizer
-from nltk.tokenize import TweetTokenizer, RegexpTokenizer
+from nltk.tokenize import TweetTokenizer
 from bs4 import BeautifulSoup
 import numpy as np
-from itertools import combinations
-from gensim.models import Phrases
 from collections import Counter
 import regex as re
 
@@ -310,7 +282,6 @@ def return_stop_words():
     add_words = ["s", "m",'you', 'not',  'get', 'no', 'via', 'one', 'still', 'us', 'u','hey','hi','oh','jeez',
                 'the', 'a', 'in', 'to', 'of', 'i', 'and', 'is', 'for', 'on', 'it', 'got','aww','awww',
                 'not', 'my', 'that', 'by', 'with', 'are', 'at', 'this', 'from', 'be', 'have', 'was']
-    from sklearn.feature_extraction import text
     #stopWords = text.ENGLISH_STOP_WORDS.union(add_words)
     stop_words = set(set(stopwords.words('english')).union(add_words))
     excl =['will',"i'll",'shall',"you'll",'may',"don't","hadn't","hasn't","haven't",
@@ -330,7 +301,7 @@ pattern = r"(?u)\b\w\w+\b"
 
 lemmatizer = WordNetLemmatizer()
 
-punc = list(set(string.punctuation))
+punc = list(set(string.punctuation))+['/;','//']
 
 def casual_tokenizer(text): #Splits words on white spaces (leaves contractions intact) and splits out trailing punctuation
     tokens = tokenizer.tokenize(text)
@@ -384,11 +355,11 @@ def process_text(text):
     decontract = [expandContractions(item, c_re=c_re) for item in lower]
     tagged = nltk.pos_tag(decontract)
     lemma = lemma_wordnet(tagged)
-    no_num = [re.sub('[0-9]+', '', each) for each in lemma]
-    no_punc = [w for w in no_num if w not in punc]
+    #no_num = [re.sub('[0-9]+', '', each) for each in lemma]
+    no_punc = [w for w in lemma if w not in punc]
     no_stop = [w for w in no_punc if w not in stop_words]
     return no_stop
-################################################################################
+################################################################################################################################################################
 ####   THE ABOVE Process_Text secion Re-used with Permission from:
 ####  R O B   S A L G A D O    robert.salgado@gmail.com Thank YOU!
 ################################################################################
@@ -437,7 +408,6 @@ def remove_punctuations(text):
     return re.sub(r'[.|,\'|,|)|(|\|/|+|-|{|}|]', r' ', remove_puncs)
 
 ################################################################################
-import collections
 def print_top_feature_grams(X, vectorizer, top_n = 200):
     """
     This prints the top features by each n-gram using the vectorizer that is selected as best!
@@ -486,7 +456,6 @@ def remove_unicode_strings(lst):
     except:
         return lst
 ###########################################################################################
-import pdb
 def print_top_features(train,nlp_column, best_nlp_vect, target, top_nums=200):
     """
     #### This can be done only for C L A S S I F I C A T I O N   Data Sets ####################
@@ -551,8 +520,7 @@ def plot_confusion_matrix(y_test,y_pred, model_name='Model'):
     pd.options.display.float_format = '{:,.2f}'.format
 
     #Get the confusion matrix and put it into a df
-    from sklearn.metrics import accuracy_score, precision_score, recall_score
-    from sklearn.metrics import f1_score, roc_curve, auc, confusion_matrix
+    from sklearn.metrics import confusion_matrix, f1_score
 
     cm = confusion_matrix(y_test, y_pred)
 
@@ -593,8 +561,8 @@ def plot_classification_matrix(y_test, y_pred, model_name='Model'):
     pd.options.display.float_format = '{:,.2f}'.format
 
     #Get the confusion matrix and put it into a df
-    from sklearn.metrics import accuracy_score, precision_score, recall_score
-    from sklearn.metrics import f1_score, roc_curve, auc, confusion_matrix,classification_report
+    from sklearn.metrics import precision_score
+    from sklearn.metrics import classification_report
     from sklearn.metrics import precision_score
 
     cm = classification_report(y_test, y_pred,output_dict=True)
@@ -637,6 +605,9 @@ def select_top_features_from_SVD(X, tsvd, is_train=True, top_n=100):
     ####   then you select the top 25 terms in 1-gram and 2-gram that make sense.
     print('Reducing dimensions from %d term-matrix to %d dimensions using TruncatedSVD...' %(X.shape[1],top_n))
     if is_train:
+        if X.shape[1] < top_n:
+            ### Sometimes there are not even 100 columns after using TFIDF, so better to cut it short.
+            top_n = int(X.shape[1] - 1)
         tsvd = TruncatedSVD(n_components=top_n,
                    n_iter=10,
                    random_state=3)
@@ -747,9 +718,8 @@ def tokenize_and_stem(text):
     stems = [stemmer.stem(t) for t in filtered_tokens]
     return stems
 ################################################################################
-from sklearn.feature_extraction import text
 from nltk.stem.snowball import SnowballStemmer
-from nltk.tokenize import TweetTokenizer, RegexpTokenizer
+from nltk.tokenize import TweetTokenizer
 ################################################################################
 def select_best_nlp_vectorizer(model, data, col, target, metric,
                     seed, modeltype,min_df):
@@ -795,7 +765,7 @@ def select_best_nlp_vectorizer(model, data, col, target, metric,
     X_train,X_test,y_train,y_test = train_test_split(data[col],
                                     data[target],test_size=0.2,random_state=seed)
     max_features_high = int(250000000/X_train.shape[0])
-    print('    However max_features limit = %d will limit numerous features from being generated' %max_features_high)
+    print('    However max_features limit = %d will limit too many features from being generated' %max_features_high)
     best_vec = None
     all_vecs = {}
     all_models = {}
@@ -866,7 +836,6 @@ def select_best_nlp_vectorizer(model, data, col, target, metric,
                                              vect_lemma, seed, modeltype)
     except:
         print('Error: Using CountVectorizer')
-
     print('\n# Using TFIDF vectorizer with binary=True, ngram = (1,3) and max_features=%d' %max_features_high)
     ##### This is based on artificially setting 5GB as being max memory limit for the term-matrix
     tvec = TfidfVectorizer( max_features=max_features_high, max_df=best_max_df, token_pattern=r'\w{1,}',
@@ -876,8 +845,7 @@ def select_best_nlp_vectorizer(model, data, col, target, metric,
                                       y_test, target, metric,
                                         tvec, seed, modeltype)
     max_features_limit = int(tvec.fit_transform(data_dtm).shape[1])
-
-
+    ##### This is based on using a Latin-1 vectorizer in case Spanish words are in text
     print('\n# Using TFIDF vectorizer with latin-1 encoding, binary=False, ngram (1,3) and limited max_features')
     tvec2 = TfidfVectorizer( max_features=max_features, max_df=best_max_df,
                                 token_pattern=r'\w{1,}', sublinear_tf=True,
@@ -888,13 +856,15 @@ def select_best_nlp_vectorizer(model, data, col, target, metric,
     all_vecs[tvec2], all_models[tvec2] = tokenize_test_by_metric(model, X_train, X_test, y_train,
                                       y_test, target, metric,
                                         tvec2, seed, modeltype)
-
-    #print('\n# Using TFIDF vectorizer with Snowball Stemming, ngram (1,3) and very high max_features')
+    #Finally Using a basic count vectorizer with all defaults while limited max features
     print('\n# Finally comparing them against a Basic Count Vectorizer with all defaults, max_features = %d and  lowercase=True' %max_features_high)
     cvect = CountVectorizer(min_df=2, lowercase=True, max_features=max_features_high, binary=False)
-    all_vecs[cvect], all_models[cvect] = tokenize_test_by_metric(model, X_train, X_test, y_train,
+    try:
+        all_vecs[cvect], all_models[cvect] = tokenize_test_by_metric(model, X_train, X_test, y_train,
                                       y_test, target, metric,
                                       cvect, seed, modeltype)
+    except:
+        print('Error: Using CountVectorizer')
     ######## Once you have built 4 different transformers it is time to compare them
     if modeltype.endswith('Classification'):
         if metric in ['log_loss','logloss']:
@@ -975,12 +945,10 @@ def class_info(classes):
     for cls in counts.keys():
         print("%6s: % 7d  =  % 5.1f%%" % (cls, counts[cls], counts[cls]/total*100))
 #########################################################################################
-from xgboost.sklearn import XGBClassifier
 from xgboost.sklearn import XGBRegressor
 from sklearn.naive_bayes import GaussianNB, MultinomialNB
-import xgboost as xgb
 from sklearn.model_selection import train_test_split
-from sklearn.feature_extraction.text import CountVectorizer, TfidfTransformer, TfidfVectorizer
+from sklearn.feature_extraction.text import CountVectorizer, TfidfVectorizer
 import time
 import scipy as sp
 from sklearn.model_selection import StratifiedKFold, KFold
@@ -1086,7 +1054,10 @@ def Auto_NLP(nlp_column, train, test, target, score_type='',
     print('    Cleaning text in %s before doing transformation...' %nlp_column)
     import time
     start_time1 = time.time()
-    print('Cleaning text in Train data. This will take a long time for large >100K data sets...' )
+    if train.shape[0] >= 100000:
+        print('Cleaning text in Train data. Please be patient since this is a large dataset with >100K rows...' )
+    else:
+        print('Cleaning text in Train data')
     train[nlp_column] = train[nlp_column].apply(expand_text).apply(remove_punctuations)
     print('    Time Taken for Expanding text in Train data = %0.0f seconds' %(time.time()-start_time1) )
     start_time2 = time.time()
@@ -1101,7 +1072,10 @@ def Auto_NLP(nlp_column, train, test, target, score_type='',
         print('    Cleaning text in %s before doing transformation...' %nlp_column)
         import time
         start_time1 = time.time()
-        print('Cleaning text in Test data. This will take a long time for large >100K data sets...' )
+        if test.shape[0] >= 100000:
+            print('Cleaning text in Test data. Please be patient since this is a large dataset with >100K rows...' )
+        else:
+            print('Cleaning text in Test data')
         test[nlp_column] = test[nlp_column].apply(expand_text).apply(remove_punctuations)
         print('    Time Taken for Expanding text in Test data = %0.0f seconds' %(time.time()-start_time1) )
         start_time2 = time.time()
@@ -1182,12 +1156,12 @@ def Auto_NLP(nlp_column, train, test, target, score_type='',
         print('    Train Vectorized data shape = %s, Cross Validation data shape = %s' %(X_train_dtm.shape, X_test.shape))
         if modeltype == 'Regression':
             scv = KFold(n_splits=n_splits, random_state=seed)
-            from sklearn.linear_model import LassoLars
-            model_name = 'Lasso LARS'
-            nlp_model = LassoLars()
+            from sklearn.svm import LinearSVR
+            model_name = 'Linear_SVR'
+            nlp_model = LinearSVR(epsilon=0.0, tol=0.001, C=1.0,random_state=99)
             params = {}
-            params['alpha'] = sp.stats.uniform(scale=1)
-            params['max_iter'] = sp.stats.randint(100,5000)
+            params['epsilon'] = sp.stats.uniform(scale=1)
+            params['C'] = sp.stats.uniform(scale=100)
         else:
             scv = StratifiedKFold(n_splits=n_splits, shuffle=True, random_state=seed)
             model_name = 'Multinomial Naive Bayes'
@@ -1196,8 +1170,12 @@ def Auto_NLP(nlp_column, train, test, target, score_type='',
             params['alpha'] = sp.stats.uniform(scale=1)
         gs = RandomizedSearchCV(nlp_model,params, n_iter=10, cv=scv,
                                 scoring=score_type, random_state=seed)
-        gs.fit(X_train_dtm,y_train)
-        y_pred = gs.predict(X_test_dtm)
+        try:
+            gs.fit(X_train_dtm,y_train)
+            y_pred = gs.predict(X_test_dtm)
+        except:
+            gs.fit(X_train_dtm.toarray(),y_train)
+            y_pred = gs.predict(X_test_dtm.toarray())
         ##### Print the model results on Cross Validation data set (held out)
         if modeltype == 'Regression':
             print_regression_model_stats(y_test, y_pred,'%s Model: Predicted vs Actual for %s' %(model_name,target))
@@ -1207,7 +1185,11 @@ def Auto_NLP(nlp_column, train, test, target, score_type='',
         #### Now select the best estimator from the RandomizedSearchCV models
         nlp_model = gs.best_estimator_
         #### Build a pipeline with the best estimator and the best vectorizer together here!
-        pipe = make_pipeline(cvect,nlp_model)
+        from sklearn.preprocessing import FunctionTransformer
+        pipe = make_pipeline(
+             cvect,
+             FunctionTransformer(lambda x: x.todense(), accept_sparse=True),
+             nlp_model)
         ### Train the Pipeline on the full data set !
         print('Training Pipeline on full Train data. This will take time...')
         #####  Now AFTER TRAINING, make predictions on the given test data set!
@@ -1304,8 +1286,7 @@ def plot_confusion_matrix(y_test,y_pred, model_name='Model'):
     pd.options.display.float_format = '{:,.2f}'.format
 
     #Get the confusion matrix and put it into a df
-    from sklearn.metrics import accuracy_score, precision_score, recall_score
-    from sklearn.metrics import f1_score, roc_curve, auc, confusion_matrix
+    from sklearn.metrics import confusion_matrix, f1_score
 
     cm = confusion_matrix(y_test, y_pred)
 
@@ -1346,8 +1327,8 @@ def plot_classification_matrix(y_test, y_pred, model_name='Model'):
     pd.options.display.float_format = '{:,.2f}'.format
 
     #Get the confusion matrix and put it into a df
-    from sklearn.metrics import accuracy_score, precision_score, recall_score
-    from sklearn.metrics import f1_score, roc_curve, auc, confusion_matrix,classification_report
+    from sklearn.metrics import precision_score
+    from sklearn.metrics import classification_report
     from sklearn.metrics import precision_score
 
     cm = classification_report(y_test, y_pred,output_dict=True)
@@ -1373,19 +1354,13 @@ def plot_classification_matrix(y_test, y_pred, model_name='Model'):
     plt.show();
 #################################################################################
 import time
-from sklearn.neighbors import KNeighborsClassifier, KNeighborsRegressor
 from sklearn.model_selection import StratifiedKFold
-from sklearn.preprocessing import StandardScaler
-from sklearn.ensemble import RandomForestClassifier, RandomForestRegressor, BaggingRegressor
-from sklearn.metrics import roc_auc_score, classification_report, confusion_matrix
-from sklearn.metrics import f1_score, precision_score, recall_score,accuracy_score,make_scorer
-from sklearn.model_selection import train_test_split, StratifiedShuffleSplit, KFold, StratifiedKFold
+from sklearn.metrics import roc_auc_score
+from sklearn.metrics import f1_score, precision_score, recall_score
+from sklearn.model_selection import KFold, StratifiedKFold, train_test_split
 from sklearn.metrics import mean_squared_error, mean_absolute_error
-from sklearn.preprocessing import LabelEncoder
 #### For Classification problems
 from sklearn.naive_bayes import MultinomialNB
-from sklearn.linear_model import LogisticRegression
-from sklearn import metrics
 def NLP_select_best_model_fit_predict(X, y, test, modeltype, score_type):
     """
     ###############################################################################################
@@ -1659,7 +1634,7 @@ def plot_nlp_column(df_col, label_title, ax,color='r'):
 #############################################################################
 import matplotlib.pyplot as plt
 import seaborn as sns
-from itertools import cycle, combinations
+from itertools import cycle
 def draw_dist_plots_summary_cols(df_train, target, summary_cols):
     colors = cycle('byrcmgkbyrcmgkbyrcmgkbyrcmgkbyr')
     target_names = np.unique(df_train[target])
@@ -1694,8 +1669,8 @@ def plot_histogram_probability(dist_train, dist_test, label_title):
     plt.show();
 ########################################################################
 module_type = 'Running' if  __name__ == "__main__" else 'Imported'
-version_number = '0.0.37'
-print("""Imported Auto_NLP version: %s.. Call using:
+version_number = '0.0.39'
+print("""\nImported Auto_NLP version: %s.. Call using:
      train_nlp, test_nlp, nlp_pipeline, predictions = Auto_NLP(
                 nlp_column, train, test, target, score_type='balanced_accuracy',
                 modeltype='Classification',top_num_features=200, verbose=0,
